<?php

namespace Ems\Controllers;

use Ems\Services\ResumoService;
use Ems\Services\ApuracaoService;
use Ems\Util\ExpectedException;
use Folha\Logic\ControlePonto\Apuracao;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\{ Message, Error };

class ResumoController {
    /** @var ResumoService */
    protected $resumoService;

    /** @var ApuracaoService */
    protected $apuracaoService;

    public function __construct(
        ResumoService   $resumoService,
        ApuracaoService $apuracaoService
    ) {
        $this->resumoService    = $resumoService;
        $this->apuracaoService  = $apuracaoService;
    }

    public function getSituacoesFreq(
        DTO\Request\Row $request,
        DTO\Response $response
    ) {
        try {
            $situacoes = $this->resumoService->getSituacoesFreq();

            $response->addDataSet(new DataSet('getSituacoesFreq', $situacoes));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getResumoFreq(
        DTO\Request\Row $request,
        DTO\Response $response
    ) {
        try {
            $row = $request->getRow();
            $nrVinculo = (int) $row['NRVINCULOM'];
            $date = (string) $row['DATE'];

            $resumo = $this->apuracaoService->getResumoDiaSituFreqVinculo(
                $nrVinculo, $date
            );

            $response->addDataSet(new DataSet('getResumoFreq', $resumo));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function saveResumoFreq(
        DTO\Request\Row $request,
        DTO\Response $response
    ) {
        try {
            $row = $request->getRow();
            $cdOperador = (string) $row['CDOPERADOR'];
            $nrVinculo = (int) $row['NRVINCULOM'];
            $nrPeriodo = (int) $row['NRPERIODOAPURACAO'];
            $date = (string) $row['DATE'];
            $rows = (array) $row['rows'];

            $this->resumoService->saveResumoFreq(
                $nrVinculo,
                $nrPeriodo,
                $cdOperador,
                $date,
                $rows
            );

            $dataSet = [
                'success' => true,
            ];

            $response->addDataSet(new DataSet('saveResumoFreq', $dataSet));
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function gestorCanEditResumo(
        DTO\Request\Row $request,
        DTO\Response $response
    ) {
        try {
            $canEdit = $this->resumoService->gestorCanEditResumo();

            $dataSet = [
                'canEdit' => $canEdit,
            ];

            $response->addDataSet(new DataSet('gestorCanEditResumo', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }
}