/* Tabela TABEVEND: TABELA DE VENDA */
/* Descrio: A trigger insere registros na tabela LOGOPERAVND a partir de alteraes na tabela TABEVEND */
/* Objetivo: Gerar um log que vai registrar as alteraes na TABEVEND,
             utilizando a tabela de Logs de Operaes do VND*/
IF EXISTS (SELECT * FROM SYSOBJECTS O, SYSUSERS U
           WHERE O.NAME = 'LOG_VND_TABEVEND' AND
                 TYPE   = 'TR' 	               AND
	              U.UID  = O.UID           AND
	              U.NAME = USER)
DROP TRIGGER LOG_VND_TABEVEND
GO


CREATE TRIGGER LOG_VND_TABEVEND
ON TABEVEND
FOR INSERT, UPDATE, DELETE
AS
  DECLARE @V_DSREMOTEADDR VARCHAR(15);
  DECLARE @V_DSREMOTEHOST VARCHAR(15);
  DECLARE @V_DSSERVERRDM  VARCHAR(20);
  DECLARE @V_DSSERVMETHOD VARCHAR(30);
  DECLARE @V_DSREMOTEOPER VARCHAR(12);
  DECLARE @V_DESC         VARCHAR(500);
  DECLARE @V_ITPESQ       VARCHAR(40);
  DECLARE @V_NMOPERADOR   VARCHAR(40);
  DECLARE @V_IDTIPOOPER   VARCHAR(1);

/*VARIVEIS DO CURSOR C_CURSOR*/

  DECLARE @V_ID_COMANDO    VARCHAR(1);

  DECLARE @V_NEW_CDFILIAL            varchar(4);
  DECLARE @V_NEW_CDTABEPREC          varchar(3);
  DECLARE @V_NEW_DTINIVGPREC         datetime;
  DECLARE @V_NEW_DTFINVGPREC         datetime;
  DECLARE @V_NEW_DTREAJUSTE          datetime;
  DECLARE @V_NEW_VRREAJUSTE          double precision;
  DECLARE @V_NEW_DTULTATUTBVE        datetime;
  DECLARE @V_NEW_IDSITVGPREC         VARCHAR(1);

  DECLARE @V_OLD_CDFILIAL            varchar(4);
  DECLARE @V_OLD_CDTABEPREC          varchar(3);
  DECLARE @V_OLD_DTINIVGPREC         datetime;
  DECLARE @V_OLD_DTFINVGPREC         datetime;
  DECLARE @V_OLD_DTREAJUSTE          datetime;
  DECLARE @V_OLD_VRREAJUSTE          double precision;
  DECLARE @V_OLD_DTULTATUTBVE        datetime;
  DECLARE @V_OLD_IDSITVGPREC         VARCHAR(1);

BEGIN
/* IDTIPOOPER - '1' - Incluso, '2' - Alterao, '3' - Excluso*/

   SET @V_DESC   = '';
   SET @V_ITPESQ     = '';
   SET @V_NMOPERADOR = '';
   SET @V_IDTIPOOPER = '';

   EXEC NMUSUARIO @P_NMUSUARIO = @V_NMOPERADOR OUTPUT;
   SET @V_NMOPERADOR = ISNULL(@V_NMOPERADOR, ' ');

   EXEC WEBTUNNING  @P_DSREMOTEADDR = @V_DSREMOTEADDR OUTPUT,
                    @P_DSREMOTEHOST = @V_DSREMOTEHOST OUTPUT,
                    @P_DSSERVERRDM  = @V_DSSERVERRDM OUTPUT,
                    @P_DSSERVMETHOD = @V_DSSERVMETHOD OUTPUT,
                    @P_DSREMOTEOPER = @V_DSREMOTEOPER OUTPUT;

   DECLARE C_CURSOR CURSOR FOR
     SELECT
     'D'   ID_COMANDO,
           NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,
           DELETED.CDFILIAL,
           DELETED.CDTABEPREC,
           DELETED.DTINIVGPREC,
           DELETED.DTFINVGPREC,
           DELETED.DTREAJUSTE,
           DELETED.VRREAJUSTE,
           DELETED.DTULTATUTBVE,
           DELETED.IDSITVGPREC
     FROM
           DELETED
     WHERE     
           DELETED.CDFILIAL + DELETED.CDTABEPREC NOT IN (SELECT  CDFILIAL + CDTABEPREC   FROM INSERTED)AND
           DELETED.DTINIVGPREC  NOT IN (SELECT DTINIVGPREC FROM INSERTED)
     UNION ALL

     SELECT
     'U'   IDCOMANDO,
           INSERTED.CDFILIAL,
           INSERTED.CDTABEPREC,
           INSERTED.DTINIVGPREC,
           INSERTED.DTFINVGPREC,
           INSERTED.DTREAJUSTE,
           INSERTED.VRREAJUSTE,
           INSERTED.DTULTATUTBVE,
           INSERTED.IDSITVGPREC,
           DELETED.CDFILIAL,
           DELETED.CDTABEPREC,
           DELETED.DTINIVGPREC,
           DELETED.DTFINVGPREC,
           DELETED.DTREAJUSTE,
           DELETED.VRREAJUSTE,
           DELETED.DTULTATUTBVE,
           DELETED.IDSITVGPREC
     FROM
           INSERTED,
           DELETED
     WHERE     
           INSERTED.CDFILIAL     = DELETED.CDFILIAL   AND
           INSERTED.CDTABEPREC  = DELETED.CDTABEPREC  AND
           INSERTED.DTINIVGPREC = DELETED.DTINIVGPREC
     UNION ALL

     SELECT
     'I'   IDCOMANDO, INSERTED.CDFILIAL,
           INSERTED.CDTABEPREC,
           INSERTED.DTINIVGPREC,
           INSERTED.DTFINVGPREC,
           INSERTED.DTREAJUSTE,
           INSERTED.VRREAJUSTE,
           INSERTED.DTULTATUTBVE,
           INSERTED.IDSITVGPREC,
           NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL
     FROM
           INSERTED
     WHERE     
		   INSERTED.CDFILIAL + INSERTED.CDTABEPREC  NOT IN (SELECT CDFILIAL + CDTABEPREC FROM DELETED) AND
           INSERTED.DTINIVGPREC NOT IN (SELECT DTINIVGPREC FROM DELETED)
 
     OPEN C_CURSOR;
     FETCH NEXT FROM C_CURSOR INTO @V_ID_COMANDO,
           			   @V_NEW_CDFILIAL,@V_NEW_CDTABEPREC,@V_NEW_DTINIVGPREC,@V_NEW_DTFINVGPREC,
           			   @V_NEW_DTREAJUSTE,@V_NEW_VRREAJUSTE,@V_NEW_DTULTATUTBVE,@V_NEW_IDSITVGPREC,
					   @V_OLD_CDFILIAL,@V_OLD_CDTABEPREC,@V_OLD_DTINIVGPREC,@V_OLD_DTFINVGPREC,
           			   @V_OLD_DTREAJUSTE,@V_OLD_VRREAJUSTE,@V_OLD_DTULTATUTBVE,@V_OLD_IDSITVGPREC

    WHILE (@@FETCH_STATUS = 0) BEGIN

       IF @V_ID_COMANDO= 'U' /*UPDATE*/ BEGIN
		IF ISNULL(@V_NEW_CDFILIAL,'NULL') <> ISNULL(@V_OLD_CDFILIAL,'NULL') BEGIN
			SET @V_DESC   = 'Filial ' + ISNULL(@V_NEW_CDFILIAL,'' );
		END;
		IF ISNULL(@V_NEW_CDTABEPREC,'NULL') <> ISNULL(@V_OLD_CDTABEPREC,'NULL') BEGIN
            SET @V_DESC  = ISNULL(@V_DESC,'') + '  Tab Preo ' + ISNULL(@V_NEW_CDTABEPREC,'' );
		END;
		IF ISNULL(CONVERT(VARCHAR,@V_NEW_DTINIVGPREC),'NULL' ) <> ISNULL(CONVERT(VARCHAR,@V_OLD_DTINIVGPREC),'NULL' ) BEGIN
            SET @V_DESC  = ISNULL(@V_DESC,'') + '  Dt Ini Vg Preco ' + ISNULL(CONVERT(VARCHAR,@V_NEW_DTINIVGPREC),'' );
		END;
		IF ISNULL(CONVERT(VARCHAR,@V_NEW_DTFINVGPREC),'NULL' ) <> ISNULL(CONVERT(VARCHAR,@V_OLD_DTFINVGPREC),'NULL' ) BEGIN
            SET @V_DESC  = ISNULL(@V_DESC,'') + '  Dt Fin Vg Preco  ' + ISNULL(CONVERT(VARCHAR,@V_NEW_DTFINVGPREC),'' );
		END;
		IF ISNULL(CONVERT(VARCHAR,@V_NEW_DTREAJUSTE ),'NULL'  ) <> ISNULL(CONVERT(VARCHAR,@V_OLD_DTREAJUSTE ),'NULL'  ) BEGIN
            SET @V_DESC  = ISNULL(@V_DESC,'') + '  Data Reajuste  ' + ISNULL(CONVERT(VARCHAR,@V_NEW_DTREAJUSTE ),'' );
		END;
		IF ISNULL(CONVERT(VARCHAR,@V_NEW_VRREAJUSTE),'NULL') <> ISNULL(CONVERT(VARCHAR,@V_OLD_VRREAJUSTE),'NULL') BEGIN
            SET @V_DESC  = ISNULL(@V_DESC,'') + '  Vr Reajuste ' + ISNULL(CONVERT(VARCHAR,@V_NEW_VRREAJUSTE),'' );
		END;
		IF ISNULL(CONVERT(VARCHAR,@V_NEW_DTULTATUTBVE),'NULL') <> ISNULL(CONVERT(VARCHAR,@V_OLD_DTULTATUTBVE),'NULL') BEGIN
            SET @V_DESC  = ISNULL(@V_DESC,'') + '  Dt Ult Atu Tab Venda ' + ISNULL(CONVERT(VARCHAR,@V_NEW_DTULTATUTBVE),'');
		END;
		IF ISNULL(@V_NEW_IDSITVGPREC,'NULL') <> ISNULL(@V_OLD_IDSITVGPREC,'NULL') BEGIN
            SET @V_DESC  = ISNULL(@V_DESC,'') + '  Id Sit Vg Preco ' + ISNULL(@V_NEW_IDSITVGPREC,'');
		END;
          SET @V_IDTIPOOPER = '2';
       END;
       IF @V_ID_COMANDO= 'I' /*INSERO*/ BEGIN
          SET @V_DESC   = 'Filial '           + ISNULL(@V_NEW_CDFILIAL,''    )  + ' ' +
                      'Tab Preo  '           + ISNULL(@V_NEW_CDTABEPREC,''  )  + ' ' +
                      'Dt Ini Vg Preco '      + ISNULL(CONVERT(VARCHAR,@V_NEW_DTINIVGPREC),'' )  + ' ' +
                      'Dt Fin Vg Preco  '     + ISNULL(CONVERT(VARCHAR,@V_NEW_DTFINVGPREC),'' )  + ' ' +
                      'Data Reajuste  '       + ISNULL(CONVERT(VARCHAR,@V_NEW_DTREAJUSTE ),'' )  + ' ' +
                      'Vr Reajuste '          + ISNULL(CONVERT(VARCHAR,@V_NEW_VRREAJUSTE),''  )  + ' ' +
                      'Dt Ult Atu Tab Venda ' + ISNULL(CONVERT(VARCHAR,@V_NEW_DTULTATUTBVE),'')  + ' ' +
                      'Id Sit Vg Preco '      + ISNULL(@V_NEW_IDSITVGPREC,'');
          SET @V_IDTIPOOPER = '1';
       END;
       IF @V_ID_COMANDO= 'D' /*DELEO*/  BEGIN
          SET @V_DESC   = 'Filial '           + ISNULL(@V_OLD_CDFILIAL,''    )  + ' ' +
                      'Tab Preo  '           + ISNULL(@V_OLD_CDTABEPREC,''  )  + ' ' +
                      'Dt Ini Vg Preco '      + ISNULL(CONVERT(VARCHAR,@V_OLD_DTINIVGPREC),'' )  + ' ' +
                      'Dt Fin Vg Preco  '     + ISNULL(CONVERT(VARCHAR,@V_OLD_DTFINVGPREC),'' )  + ' ' +
                      'Data Reajuste  '       + ISNULL(CONVERT(VARCHAR,@V_OLD_DTREAJUSTE ),'' )  + ' ' +
                      'Vr Reajuste '          + ISNULL(CONVERT(VARCHAR,@V_OLD_VRREAJUSTE),''  )  + ' ' +
                      'Dt Ult Atu Tab Venda ' + ISNULL(CONVERT(VARCHAR,@V_OLD_DTULTATUTBVE),'')  + ' ' +
                      'Id Sit Vg Preco '      + ISNULL(@V_OLD_IDSITVGPREC,'');
          SET @V_IDTIPOOPER = '3';
       END;
       IF  (@V_IDTIPOOPER <> '') BEGIN
          IF @V_DESC IS NOT NULL BEGIN
             SET @V_ITPESQ =  ISNULL( @V_NEW_CDFILIAL + @V_NEW_CDTABEPREC + CONVERT(VARCHAR,@V_NEW_DTINIVGPREC) , @V_OLD_CDFILIAL + @V_OLD_CDTABEPREC + CONVERT(VARCHAR,@V_OLD_DTINIVGPREC));
             SET @V_DESC = SUBSTRING(@V_DESC, 1, 300);
             INSERT INTO LOGOPERAVND
                    (NMTABELAVND, DTOPERACAOVND, NMRESPOPERVND, IDTIPOOPERVND, DSOPERACAOVND,
                     CDPESQITEMVND, DSRDMUSUOPERVND, DSENDUSUOPERVND, DSNMUSUOPERVND, DSMETUSUOPERVND,
                     CDORESPOPERVND, CDFILLOGOPERVND)
             VALUES ('TABEVEND',GETDATE(), ISNULL(@V_NMOPERADOR,' '), @V_IDTIPOOPER, @V_DESC, @V_ITPESQ,
                     @V_DSSERVERRDM, @V_DSREMOTEADDR, @V_DSREMOTEHOST, @V_DSSERVMETHOD, @V_DSREMOTEOPER,
                     ISNULL(@V_NEW_CDFILIAL,@V_OLD_CDFILIAL));
          END;
       END;
       FETCH NEXT FROM C_CURSOR INTO @V_ID_COMANDO,
           		     	     @V_NEW_CDFILIAL,@V_NEW_CDTABEPREC,@V_NEW_DTINIVGPREC,@V_NEW_DTFINVGPREC,
           			     @V_NEW_DTREAJUSTE,@V_NEW_VRREAJUSTE,@V_NEW_DTULTATUTBVE,@V_NEW_IDSITVGPREC,
				     @V_OLD_CDFILIAL,@V_OLD_CDTABEPREC,@V_OLD_DTINIVGPREC,@V_OLD_DTFINVGPREC,
            			     @V_OLD_DTREAJUSTE,@V_OLD_VRREAJUSTE,@V_OLD_DTULTATUTBVE,@V_OLD_IDSITVGPREC
    SET @V_DESC   = '';        
    END;
    CLOSE      C_CURSOR;
    DEALLOCATE C_CURSOR;
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'LOG_VND_TABEVEND','5.20.001';
GO 